@@ -33,8 +33,7 @@ void        bus_selinux_shutdown (void);
 
 dbus_bool_t bus_selinux_enabled  (void);
 
-void bus_selinux_id_ref    (BusSELinuxID *sid);
-void bus_selinux_id_unref  (BusSELinuxID *sid);
+BusSELinuxID *bus_selinux_get_self (void);
 
 DBusHashTable* bus_selinux_id_table_new    (void);
 BusSELinuxID*  bus_selinux_id_table_lookup (DBusHashTable    *service_table,
